@@ -10,7 +10,7 @@
 #include "wimlib/lzx_constants.h"
 #include "wimlib/types.h"
 
-extern const u32 lzx_offset_slot_base[LZX_MAX_OFFSET_SLOTS + 1];
+extern const s32 lzx_offset_slot_base[LZX_MAX_OFFSET_SLOTS + 1];
 
 extern const u8 lzx_extra_offset_bits[LZX_MAX_OFFSET_SLOTS];
 
